# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, Gdk # pylint: disable=E0611
import logging
logger = logging.getLogger('indicator_remindor')

from indicator_remindor.indicator_remindorconfig import get_data_file
from indicator_remindor.helpers import get_builder, show_uri, get_help_uri, database_file, get_media_file
from indicator_remindor.helperdialogs import DateDialog, TimeDialog, CommandDialog

from remindor_common.datetimeutil import str_time_simplify, str_date_simplify, fix_time_format, fix_date_format
from remindor_common import services
from remindor_common import database as db
from remindor_common.helpers import PreferencesDialogInfo

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

class PreferencesDialog(Gtk.Dialog):
    __gtype_name__ = "PreferencesDialog"

    def __new__(cls):
        builder = get_builder('PreferencesDialog')
        new_object = builder.get_object("preferences_dialog")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self, True)

        self.label_o = self.builder.get_object("label_entry")
        self.time_o = self.builder.get_object("time_entry")
        self.date_o = self.builder.get_object("date_entry")
        self.command_o = self.builder.get_object("command_entry")
        self.postpone_o = self.builder.get_object("postpone_entry")

        self.sound_o = self.builder.get_object("sound_entry")
        self.sound_length_o = self.builder.get_object("length_entry")
        self.sound_loop_o = self.builder.get_object("loop_entry")
        self.loop_times_o = self.builder.get_object("loop_times_entry")

        self.notification_o = self.builder.get_object("notification_entry")
        self.dialog_o = self.builder.get_object("dialog_entry")
        self.change_icon_o = self.builder.get_object("change_icon_entry")

        self.quick_label_o = self.builder.get_object("quick_label_entry")
        self.quick_minutes_o = self.builder.get_object("quick_minutes_entry")
        self.quick_unit_o = self.builder.get_object("quick_unit_entry")
        self.quick_unit_o.set_active(0)
        self.quick_slider_o = self.builder.get_object("quick_slider_entry")
        self.quick_popup_o = self.builder.get_object("quick_popup_entry")
        self.quick_dialog_o = self.builder.get_object("quick_dialog_entry")
        self.quick_sound_o = self.builder.get_object("quick_sound_entry")
        self.quick_info_o = self.builder.get_object("quick_info_entry")

        self.time_format_o = self.builder.get_object("time_format_entry")
        self.date_format_o = self.builder.get_object("date_format_entry")

        self.today_color_o = self.builder.get_object("today_color_entry")
        self.future_color_o = self.builder.get_object("future_color_entry")
        self.past_color_o = self.builder.get_object("past_color_entry")
        self.news_o = self.builder.get_object("news_entry")
        self.hide_indicator_o = self.builder.get_object("hide_indicator_entry")
        self.indicator_icon_o = self.builder.get_object("indicator_icon_entry")

        self.boxcar_o = self.builder.get_object("boxcar_email_entry")
        self.boxcar_notification_o = self.builder.get_object("boxcar_notification_entry")

        #not sure why this doesn't connect automagically
        self.builder.get_object("date_edit_button").connect('clicked', self.on_date_edit_button_click)

        self.label_o.connect('icon-release', self.clear_entry)
        self.quick_label_o.connect('icon-release', self.clear_entry)
        self.time_o.connect('icon-release', self.clear_entry)
        self.time_o.connect('changed', self.on_time_entry_changed)
        self.date_o.connect('icon-release', self.clear_entry)
        self.date_o.connect('changed', self.on_date_entry_changed)
        self.command_o.connect('icon-release', self.clear_entry)
        self.boxcar_o.connect('icon-release', self.clear_entry)

        self.info = PreferencesDialogInfo(database_file())
        self.settings = self.info.settings
        self.boxcar = self.info.services.service('boxcar')

        self.setup_services()
        self.setup_data()

    def clear_entry(self, widget, pos, event, data=None):
        widget.set_text("")

    def setup_services(self):
        self.boxcar_o.set_text(self.boxcar.email)
        self.boxcar_original = self.boxcar.email
        self.boxcar_notification_o.set_active(self.boxcar.default_notify)

    def setup_data(self):
        self.label_o.set_text(self.settings.label)
        self.command_o.set_text(self.settings.command)
        self.postpone_o.set_value(self.settings.postpone)

        if self.settings.sound_file is not None and not self.settings.sound_file == "":
            self.sound_o.set_filename(self.settings.sound_file)
        else:
            sound_dir = get_data_file('media', 'sounds')
            self.sound_o.set_current_folder(sound_dir)

        self.sound_length_o.set_value(self.settings.sound_length)
        self.sound_loop_o.set_active(self.settings.sound_loop)
        self.loop_times_o.set_value(self.settings.sound_loop_times)

        self.notification_o.set_active(self.settings.notification)
        self.dialog_o.set_active(self.settings.dialog)
        self.change_icon_o.set_active(self.settings.change_icon)

        self.quick_label_o.set_text(self.settings.quick_label)
        self.quick_minutes_o.set_value(self.settings.quick_minutes)
        self.quick_unit_o.set_active(self.settings.quick_unit)
        self.quick_slider_o.set_active(self.settings.quick_slider)
        self.quick_popup_o.set_active(self.settings.quick_popup)
        self.quick_dialog_o.set_active(self.settings.quick_dialog)
        self.quick_sound_o.set_active(self.settings.quick_sound)
        self.quick_info_o.set_active(self.settings.quick_info)

        self.time_format_o.set_active(self.settings.time_format)
        self.date_format_o.set_active(self.settings.date_format)

        self.today_color_o.set_color(Gdk.color_parse(self.settings.today_color))
        self.future_color_o.set_color(Gdk.color_parse(self.settings.future_color))
        self.past_color_o.set_color(Gdk.color_parse(self.settings.past_color))
        self.news_o.set_active(self.settings.show_news)
        self.hide_indicator_o.set_active(self.settings.hide_indicator)
        self.hide_start = self.settings.hide_indicator
        self.indicator_icon_o.set_active(self.settings.indicator_icon)

        self.time_o.set_text(self.settings.time)
        self.date_o.set_text(self.settings.date)

    def on_date_entry_changed(self, widget, data=None):
        save = self.builder.get_object("btn_save")
        if str_date_simplify(widget.get_text(), self.settings.date_format) == None:
            save.set_sensitive(False)
            widget.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
        else:
            if not str_time_simplify(self.time_o.get_text()) == None:
                save.set_sensitive(True)
            widget.set_icon_from_stock(0, None)

    def on_time_entry_changed(self, widget, data=None):
        save = self.builder.get_object("btn_save")
        if str_time_simplify(widget.get_text()) == None:
            save.set_sensitive(False)
            widget.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
        else:
            if not str_date_simplify(self.date_o.get_text(), self.settings.date_format) == None:
                save.set_sensitive(True)
            widget.set_icon_from_stock(0, None)

    def on_sound_clear_clicked(self, widget, data=None):
        self.sound_o.unselect_all()
        sound_dir = get_data_file('media', 'sounds')
        self.sound_o.set_current_folder(sound_dir)

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()

    def on_btn_save_clicked(self, widget, data=None):
        service_status = self.save_services()

        if service_status:
            hide = self.hide_indicator_o.get_active()
            ok = True
            if hide and not self.hide_start:
                message = _("You have chosen to hide the indicator.\nOnly use this option if you know what you are doing!\n\nYou can run the command \"%s -p\"\nfrom the command line to open the preferences dialog\nto change this option.\n\nWould you like to continue?") % "indicator-remindor"
                dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.WARNING,
                                    Gtk.ButtonsType.YES_NO, message)
                response = dialog.run()
                dialog.destroy()

                if response == -9: #they said No
                    ok = False
                    self.hide_indicator_o.set_active(False)

            if ok:
                self.settings.label = self.label_o.get_text()
                #time = str_time_simplify(self.time_o.get_text())
                #same as below
                self.settings.time = self.time_o.get_text()
                #date = str_date_simplify(self.date_o.get_text())
                #actually don't do this because then they can keep "today" as their option
                #AlarmDialog will process it further
                self.settings.date = self.date_o.get_text()
                self.settings.command = self.command_o.get_text()
                self.settings.postpone = self.postpone_o.get_value()

                self.settings.sound_file = self.sound_o.get_filename()
                self.settings.sound_length = self.sound_length_o.get_value()
                self.settings.sound_loop = self.sound_loop_o.get_active()
                self.settings.sound_loop_times = self.loop_times_o.get_value()

                self.settings.notification = self.notification_o.get_active()
                self.settings.dialog = self.dialog_o.get_active()
                self.settings.change_icon = self.change_icon_o.get_active()

                self.settings.quick_label = self.quick_label_o.get_text()
                self.settings.quick_minutes = self.quick_minutes_o.get_value()
                self.settings.quick_unit = self.quick_unit_o.get_active()
                self.settings.quick_slider = self.quick_slider_o.get_active()
                self.settings.quick_popup = self.quick_popup_o.get_active()
                self.settings.quick_dialog = self.quick_dialog_o.get_active()
                self.settings.quick_sound = self.quick_sound_o.get_active()
                self.settings.quick_info = self.quick_info_o.get_active()

                self.settings.time_format = self.time_format_o.get_active()
                self.settings.date_format = self.date_format_o.get_active()

                self.settings.today_color = self.today_color_o.get_color().to_string()
                self.settings.future_color = self.future_color_o.get_color().to_string()
                self.settings.past_color = self.past_color_o.get_color().to_string()
                self.settings.show_news = self.news_o.get_active()
                self.settings.hide_indicator = self.hide_indicator_o.get_active()
                self.settings.indicator_icon = self.indicator_icon_o.get_active()

                status = self.info.preferences(self.settings, self.boxcar)
                if status == self.info.time_error:
                    self.time_o.get_focus()
                elif status == self.info.date_error:
                    self.date_o.get_focus()
                else:
                    self.destroy()

    def save_services(self):
        return_status = True
        boxcar_email = self.boxcar_o.get_text()
        boxcar_notify = self.boxcar_notification_o.get_active()

        status = self.info.validate_boxcar(boxcar_email, boxcar_notify, self.boxcar_original)
        if status == self.info.boxcar_add:
            self.boxcar.email = boxcar_email
            self.boxcar.default_notify = boxcar_notify
        elif status == self.info.boxcar_delete:
            self.boxcar.email = ""
            self.boxcar.default_notify = False
        elif status == self.info.boxcar_subscribe:
            message = _("It seems that you are not yet signed up for Boxcar.\nPlease visit boxcar.io to signup.")
            dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.WARNING,
                                       Gtk.ButtonsType.OK, message)
            dialog.run()
            dialog.destroy()

            return_status = False
        else:
            message = _("There has been a error connecting with Boxcar,\nplease check your email address or network connection.")
            dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR,
                                       Gtk.ButtonsType.OK, message)
            dialog.run()
            dialog.destroy()

            return_status = False

        return return_status

    def on_btn_help_clicked(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri('preferences'))

    def on_time_edit_button_clicked(self, widget, data=None):
        dialog = TimeDialog()
        dialog.set_transient_for(self)
        dialog.set_data(fix_time_format(str_time_simplify(self.time_o.get_text()), self.settings.time_format))
        dialog.connect('time_updated', self.time_updated)
        dialog.run()

    def time_updated(self, widget, data=None):
        self.time_o.set_text(data)

    def on_date_edit_button_click(self, widget, data=None):
        dialog = DateDialog()
        dialog.set_transient_for(self)
        dialog.set_date_format(self.settings.date_format)
        dialog.set_data(fix_date_format(str_date_simplify(self.date_o.get_text(), self.settings.date_format), self.settings.date_format))
        dialog.connect('date_updated', self.date_updated)
        dialog.run()

    def date_updated(self, widget, data=None):
        self.date_o.set_text(data)

    def on_command_edit_button_clicked(self, widget, data=None):
        dialog = CommandDialog()
        dialog.set_transient_for(self)
        dialog.set_command(self.command_o.get_text())
        dialog.connect('command', self.command_updated)

        response = dialog.run()
        while response == -11:
            response = dialog.run()

    def command_updated(self, widget, data=None):
        self.command_o.set_text(data)
