# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject # pylint: disable=E0611

from indicator_remindor.indicator_remindorconfig import get_data_file
from indicator_remindor import helpers
from indicator_remindor.helperdialogs import DateDialog, TimeDialog, CommandDialog

from remindor_common.datetimeutil import str_date_simplify, str_time_simplify, fix_time_format, fix_date_format
from remindor_common import database as db
from remindor_common.helpers import ReminderDialogInfo, insert_values, valid_date, valid_time

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

class ReminderDialog(Gtk.Dialog):
    __gtype_name__ = "ReminderDialog"
    __gsignals__ = {
        'reminder_added' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    delete_id = -1

    def __new__(cls):
        builder = helpers.get_builder('ReminderDialog')
        new_object = builder.get_object('reminder_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.info = ReminderDialogInfo(helpers.database_file())

        #reminder entries
        self.label_e = self.builder.get_object("label_entry")
        self.time_e = self.builder.get_object("time_entry")
        self.date_e = self.builder.get_object("date_entry")
        self.command_e = self.builder.get_object("command_entry")
        self.notes_e = self.builder.get_object("notes_entry")

        self.insert_b = self.builder.get_object("insert_button")
        self.insert_e = self.builder.get_object("insert_entry")
        self.insert_e.set_active(0)

        #notification entries
        self.popup_e = self.builder.get_object("popup_entry")
        self.dialog_e = self.builder.get_object("dialog_entry")
        self.boxcar_e = self.builder.get_object("boxcar_entry")

        #notification labels
        self.boxcar_l = self.builder.get_object("boxcar_info_label")

        #sound entries
        self.play_e = self.builder.get_object("play_entry")
        self.sound_e = self.builder.get_object("sound_entry")
        self.sound_clear = self.builder.get_object("sound_clear")
        self.length_e = self.builder.get_object("length_entry")
        self.loop_e = self.builder.get_object("loop_entry")

        #sound labels
        self.loop_label = self.builder.get_object("loop_label2")

        #buttons
        self.help_b = self.builder.get_object("btn_help")
        self.cancel_b = self.builder.get_object("btn_cancel")
        self.add_b = self.builder.get_object("btn_add")

        #connections
        self.play_e.connect('notify::active', self.sound_switch)
        self.loop_e.connect('notify::active', self.sound_loop_switch)
        self.date_e.connect('icon-release', self.clear_entry)
        self.date_e.connect('changed', self.validate)
        self.time_e.connect('icon-release', self.clear_entry)
        self.time_e.connect('changed', self.validate)
        self.command_e.connect('icon-release', self.clear_entry)

        self.set_data(self.info.label, self.info.time, self.info.date, self.info.command,
                      "", self.info.popup, self.info.dialog, self.info.boxcar,
                      self.info.sound_file, self.info.sound_length, self.info.sound_loop)

        if self.info.boxcar_ok:
            self.boxcar_l.hide()
            self.boxcar_e.set_sensitive(True)
        else:
            self.boxcar_l.show()
            self.boxcar_e.set_sensitive(False)

        self.loop_label.set_text(_("(will loop %s times)") % str(self.info.sound_loop_times))

    def edit(self, id):
        self.add_b.set_label(_("Save"))
        image = Gtk.Image()
        image.set_from_stock(Gtk.STOCK_SAVE, Gtk.IconSize.BUTTON)
        self.add_b.set_image(image)

        window = self.builder.get_object('reminder_dialog')
        window.set_title(_("Edit Reminder"))

        self.database = db.Database(helpers.database_file())
        a = self.database.alarm(id)
        self.database.close()

        self.set_data(a.label, fix_time_format(a.time, self.info.time_format),
            fix_date_format(a.date, self.info.date_format), a.command, a.notes,
            a.notification, a.dialog, a.boxcar, a.sound_file, a.sound_length, a.sound_loop)

        self.delete_id = id

    def set_data(self, label, time, date, command, notes, popup,
            dialog, boxcar, sound_file, length, loop):
        self.label_e.set_text(label)
        self.time_e.set_text(time)
        self.date_e.set_text(date)
        self.command_e.set_text(command)
        buffer = self.notes_e.get_buffer()
        buffer.set_text(notes)

        self.popup_e.set_active(popup)
        self.dialog_e.set_active(dialog)
        self.boxcar_e.set_active(boxcar)

        if sound_file is not None and not sound_file == "":
            self.sound_e.set_filename(sound_file)
            self.play_e.set_active(True)
        else:
            self.sound_e.unselect_all()
            sound_dir = get_data_file('media', 'sounds')
            self.sound_e.set_current_folder(sound_dir)
            self.play_e.set_active(False)

        self.length_e.set_value(length)
        self.loop_e.set_active(loop)

        self.sound_loop_switch(None)
        self.sound_switch(None)

    def validate(self, widget, data=None):
        time_s = self.time_e.get_text()
        date_s = self.date_e.get_text()
        sensitive = True

        if not valid_time(time_s):
            self.time_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            sensitive = False
        else:
            self.time_e.set_icon_from_stock(0, None)

        if not valid_date(date_s, self.info.date_format):
            self.date_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            sensitive = False
        else:
            self.date_e.set_icon_from_stock(0, None)

        self.add_b.set_sensitive(sensitive)

    def clear_entry(self, widget, pos, event, data=None):
        widget.set_text("")

    def on_insert_button_clicked(self, widget, data=None):
        active = self.insert_e.get_active()

        buffer = self.notes_e.get_buffer()
        buffer.insert_at_cursor(insert_values[active])

    def on_sound_clear_clicked(self, widget, data=None):
        self.sound_e.unselect_all()
        sound_dir = get_data_file('media', 'sounds')
        self.sound_e.set_current_folder(sound_dir)

    def sound_loop_switch(self, widget, data=None):
        self.length_e.set_sensitive(not self.loop_e.get_active())
        if self.loop_e.get_active():
            self.loop_label.show()
        else:
            self.loop_label.hide()

    def sound_switch(self, widget, data=None):
        sensitive = False
        if self.play_e.get_active():
            sensitive = True

        self.sound_e.set_sensitive(sensitive)
        self.sound_clear.set_sensitive(sensitive)
        self.length_e.set_sensitive(sensitive)
        self.loop_e.set_sensitive(sensitive)

    def on_delete_event(self, widget, data=None):
        self.destroy()

    def on_btn_add_clicked(self, widget, data=None):
        #reminder entries
        label = self.label_e.get_text()
        time = self.time_e.get_text()
        date = self.date_e.get_text()
        command = self.command_e.get_text()
        buffer = self.notes_e.get_buffer()
        notes = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(), True)

        #notification entries
        popup = self.popup_e.get_active()
        dialog = self.dialog_e.get_active()
        boxcar = self.boxcar_e.get_active()

        #sound entries
        play = self.play_e.get_active()
        sound_file = self.sound_e.get_filename()
        sound_length = self.length_e.get_value()
        sound_loop = self.loop_e.get_active()

        if not play:
            sound_file = ""
        if sound_loop:
            sound_length = 0

        (status, id) = self.info.reminder(label, time, date, command, notes, popup, dialog,
                                          boxcar, play, sound_file, sound_length, sound_loop,
                                          self.delete_id, True)

        if status == self.info.ok:
            self.emit('reminder_added', id)
            self.destroy()
        else:
            if status == self.info.file_error:
                title = _("File does not exist")
                message = ""
                if sound_file != "" and sound_file != None:
                    message = "%s\n\n%s" % (_("The following file does not exist.\nPlease choose another sound file."), sound_file)
                else:
                    message = _("Please choose a sound file.")

                message_dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL,
                    Gtk.MessageType.WARNING, Gtk.ButtonsType.OK, message)
                message_dialog.run()
                message_dialog.destroy()
            elif status == self.info.time_error:
                self.validate()
            elif status == self.info.date_error:
                self.validate()
            elif status == self.info.notify_warn:
                title = "Boxcar"
                message = _("The label and notes for this reminder are empty,\nwould you still like to use a notification?")
                message_dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL,
                    Gtk.MessageType.WARNING, Gtk.ButtonsType.YES_NO, message)
                ans = message_dialog.run()
                message_dialog.destroy()

                if ans == -8:
                    (status, id) = self.info.reminder(label, time, date, command, notes,
                                                      popup, dialog, boxcar, play,
                                                      sound_file, sound_length, sound_loop,
                                                      self.delete_id)
                    #already checked the status (boxcar is the last check)
                    self.emit('reminder_added', id)
                    self.destroy()

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()

    def on_btn_help_clicked(self, widget, data=None):
        helpers.show_uri(self, "ghelp:%s" % helpers.get_help_uri('add'))

    def on_time_edit_button_clicked(self, widget, data=None):
        dialog = TimeDialog()
        dialog.set_transient_for(self)
        dialog.set_data(fix_time_format(str_time_simplify(self.time_e.get_text()), self.info.time_format))
        dialog.connect('time_updated', self.time_updated)
        dialog.run()

    def time_updated(self, widget, data=None):
        self.time_e.set_text(data)

    def on_date_edit_button_clicked(self, widget, data=None):
        dialog = DateDialog()
        dialog.set_transient_for(self)
        dialog.set_date_format(self.info.date_format)
        dialog.set_data(fix_date_format(str_date_simplify(self.date_e.get_text(), self.info.date_format), self.info.date_format))
        dialog.connect('date_updated', self.date_updated)
        dialog.run()

    def date_updated(self, widget, data=None):
        self.date_e.set_text(data)

    def on_command_edit_button_clicked(self, widget, data=None):
        dialog = CommandDialog()
        dialog.set_transient_for(self)
        dialog.set_command(self.command_e.get_text())
        dialog.connect('command', self.command_updated)

        response = dialog.run()
        while response == -11:
            response = dialog.run()

    def command_updated(self, widget, data=None):
        self.command_e.set_text(data)

if __name__ == "__main__":
    dialog = ReminderDialog()
    dialog.show()
    Gtk.main()
