# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""Helpers for an Ubuntu application."""
import logging
import os
import sys
import sqlite3

from indicator_remindor.indicator_remindorconfig import get_data_file
from indicator_remindor.Builder import Builder
from remindor_common import database as db

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

def get_builder(builder_file_name):
    """Return a fully-instantiated Gtk.Builder instance from specified ui
    file

    :param builder_file_name: The name of the builder file, without extension.
        Assumed to be in the 'ui' directory under the data path.
    """
    # Look for the ui file that describes the user interface.
    ui_filename = get_data_file('ui', '%s.ui' % (builder_file_name,))
    if not os.path.exists(ui_filename):
        ui_filename = None

    builder = Builder()
    builder.set_translation_domain('indicator-remindor')
    builder.add_from_file(ui_filename)
    return builder


# Owais Lone : To get quick access to icons and stuff.
def get_media_file(media_file_name):
    media_filename = get_data_file('media', '%s' % (media_file_name,))
    if not os.path.exists(media_filename):
        media_filename = None

    return "file:///" + media_filename

class NullHandler(logging.Handler):
    def emit(self, record):
        pass

def set_up_logging(opts):
    # add a handler to prevent basicConfig
    root = logging.getLogger()
    null_handler = NullHandler()
    root.addHandler(null_handler)

    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(funcName)s() '%(message)s'")

    logger = logging.getLogger('indicator_remindor')
    lib_logger = logging.getLogger('remindor_common')

    log = log_file()
    if opts.add or opts.quick or opts.manage or opts.prefs or opts.stop:
        log = config_dir() + "/indicator-remindor_console.log"
    else:
        if os.path.isfile(log_file()):
            os.remove(log_file()) #delete old log file

    logger_fh = logging.FileHandler(log)
    logger_fh.setFormatter(formatter)
    logger.addHandler(logger_fh)
    logger.setLevel(logging.DEBUG)

    lib_logger_fh = logging.FileHandler(log)
    lib_logger_fh.setFormatter(formatter)
    lib_logger.addHandler(lib_logger_fh)
    lib_logger.setLevel(logging.DEBUG)

    # Set the logging level to show debug messages.
    if opts.verbose:
        logger_sh = logging.StreamHandler()
        logger_sh.setFormatter(formatter)
        logger.addHandler(logger_sh)
        logger.debug('verbose mode')
    if opts.verbose > 1:
        lib_logger_sh = logging.StreamHandler()
        lib_logger_sh.setFormatter(formatter)
        lib_logger.addHandler(lib_logger_sh)
        lib_logger.debug('extra verbose mode')

def get_help_uri(page=None):
    # help_uri from source tree - default language
    here = os.path.dirname(__file__)
    help_uri = os.path.abspath(os.path.join(here, '..', 'help', 'C'))

    if not os.path.exists(help_uri):
        # installed so use gnome help tree - user's language
        help_uri = 'indicator-remindor'

    # unspecified page is the index.page
    if page is not None:
        help_uri = '%s#%s' % (help_uri, page)

    return help_uri

def show_uri(parent, link):
    from gi.repository import Gtk # pylint: disable=E0611
    screen = parent.get_screen()
    Gtk.show_uri(screen, link, Gtk.get_current_event_time())

def alias(alternative_function_name):
    '''see http://www.drdobbs.com/web-development/184406073#l9'''
    def decorator(function):
        '''attach alternative_function_name(s) to function'''
        if not hasattr(function, 'aliases'):
            function.aliases = []
        function.aliases.append(alternative_function_name)
        return function
    return decorator

def check_autostart():
    filename = os.getenv('HOME') + '/.config/autostart/indicator-remindor.desktop'

    file_exists = True
    try:
        with open(filename) as f: pass
        #file exists nothing to do
    except IOError:
        #file does not exists create it"""
        file_exists = False

    #find icon
    #icon = get_media_file('indicator-remindor.svg')
    #icon = icon[8:] #remove file://
    icon = "indicator-remindor"

    #find app path
    app_path = ''
    if getattr(sys, 'frozen', False):
        app_path = sys.executable
    elif __file__:
        app_path = __file__

    #find exec path
    exec_path = 'indicator-remindor'
    if '/opt/extras.ubuntu.com' in app_path:
        exec_path = '/opt/extras.ubuntu.com/indicator-remindor/bin/indicator-remindor'

    exec_exists = False #incase the file gets really messed up
    if file_exists: #make paths correct
        #code from setup.py
        fin = file(filename, 'r')
        fout = file(fin.name + '.new', 'w')

        for line in fin:
            if 'Exec=' in line:
                line = "Exec=%s\n" % exec_path
                exec_exists = True
            if 'Icon=' in line:
                line = "Icon=%s\n" % icon
            fout.write(line)
        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)

    if not file_exists or not exec_exists: #create file
        #make directory if needed
        directory = os.path.dirname(filename)
        if not os.path.exists(directory):
            os.makedirs(directory)

        #create autostart file
        f = open(filename, 'w')
        f.write('[Desktop Entry]\n')
        f.write('Name=Indicator Remindor\n')
        f.write('Comment=Indicator Remindor\n')
        f.write('Categories=GNOME;Utility;\n')
        f.write("Exec=%s\n" % exec_path)
        f.write("Icon=%s\n" % icon)
        f.write('Terminal=false\n')
        f.write('Type=Application\n')
        f.close()

def database_file():
    return os.getenv('HOME') + '/.config/indicator-remindor/indicator-remindor.db'

def config_dir():
    return os.getenv('HOME') + '/.config/indicator-remindor'

def log_file():
    return config_dir() + "/indicator-remindor.log"

def check_database():
    filename = database_file()
    database = db.Database(filename)
    database.setup()
    database.close()